require "active_query/version"
require "active_query/configuration"
require "active_query/sql_template"
require "active_query/data_adapters/postgresql_adapter"
require 'pg'

class ApplicationError < StandardError 
end

module ActiveQuery
  include ActiveQuery::DataAdapters
  # Configuration
  class << self
    attr_writer :config
  end

  def self.config
    @config ||= Configuration.new
  end

  def self.reset
    @config = Configuration.new
  end

  def self.configure
    yield(config)
  end
  
  # WIP runners... 
  # Full-Scan
  def self.run(query)
    if query.is_a? Symbol
      query = self.send(query)
    end
    con_str = ActiveQuery.config.connection
    PostgresqlAdapter.open(con_str) { |c| c.exec(query) }
  end

  def self.run_params(query, params)
    if query.is_a? Symbol
      query = self.send(query)
    end
    con_str = ActiveQuery.config.connection
    PostgresqlAdapter.open(con_str) do |conn| 
      conn.exec_params(query, params) 
    end
  end

  # async_exec(sql) {|pg_result| block } 
  # This function has the same behavior as sync_exec, but is implemented using the
  # asynchronous command processing API of libpq.
  def self.async_run(query, &block)
    con_str = ActiveQuery.config.connection
   
    if query.is_a? Symbol
      query = self.send(query)
    end

    PostgresqlAdapter.open(con_str) do |conn| 
      conn.send_query(query)
      conn.set_single_row_mode
      conn.get_result.stream_each do |row|
         yield(row)
      end
    end
  end

  # WIP Init config 
  require "active_query/config"
  include ActiveQuery::Config
  # Build heleper methods for query names
  #fail TemplatePathError, "templates path missing" if self.config.template_path.empty?
  puts "Loading SQL templates from #{self.config.template_path}" 

  begin 
    t = ActiveQuery::SQLTemplate.new(self.config)
    t.load!
    t.templates.each do |query| 
    if query
      define_singleton_method(:"#{query.name}") do |params=nil|
        query.query
      end
    end
  end
  puts "SQL templates loaded." 
  rescue ApplicationError => err
      $stderr.puts "%p ::ActiveQuery.load! %s" % [ err.class, err.message ]
  end
  
end