module ActiveQuery

	class ApplicationError < StandardError; end
	class TemplatePathError < ApplicationError; end
	class ScannerError < ApplicationError; end

	class SQLTemplate

		require_relative "sql_parser" 
		require 'erb'
		require 'find'

		attr_accessor :config
		attr_reader :templates

    def initialize(config = Configuration.new)
      @config = config
      @templates = []
    end
		
		def load!
			template_path = @config.template_path
			unless File.directory?(template_path) 
				raise TemplatePathError, "error: #{template_path} template path is invalid, please make sure it exists."
			else 
				Find.find(template_path) do |path| 
					if File.basename(path) =~ /sql$/
						sql = parse_sql(path)
						@templates << sql if sql 
					end
				end
			end
		end

		def templates
			@templates ||= [] 
		end
		
		private 
		
		def read_sql_file(path:)
			open(path, "r") { |t| ERB.new(t.read).result(binding) } 
		end

		def parse_sql(path)
			sql_template = read_sql_file(path: path) 
			begin
				ActiveQuery::SQLParser.parse_query(sql_template)
			rescue ActiveQuery::SQLParser::SyntaxError => err
			  $stderr.puts "%p %s:" % [ err.class, err.message ]
			end

		end
	end
end